unit frs_Ibase;

{*************************************************************************
                                                                          
  UNIT:           Ibase.pas                                               
  DESCRIPTION:	  OSRI entrypoints and definitions for use with Delphi    
                                                                          
  AUTHOR:         Paul Reeves                                             
                  Fleet River Software                                    
                                                                          
  Acknowledgements to InterBase Software Corporation                      
                                                                          
**************************************************************************}

interface

uses 
    Windows
  , SysUtils
  ;


const

	ISC_TRUE = 1;
	ISC_FALSE = 0;

	DSQL_close = 1;
	DSQL_drop = 2;

  IBASE_DLL = 'GDS32.DLL';

//  Ibase.h has lots of constant definitions. They cause a lot of clutter
//  when it comes to reading the function definitions, so lets keep them separate.

{$include frs_Ibase_Const.inc}


type

{*********************}
{* Fundamental types *}
{*********************}

  ISC_LONG = Longint;
	PISC_LONG = ^ISC_LONG;

  PByte     = ^Byte;
  PWord     = ^Word;
  PSmallint = ^Smallint;
  PInteger  = ^Integer;

  //short     = Smallint;     //defined in Windows.pas, usually.
  PShort    = ^Short;
  
  PLong     = ^LongInt;
  PFloat    = ^Single;
  PSingle   = ^Single;
  PDouble   = ^Double;

  ISC_QUAD = record
    isc_quad_high: ISC_LONG;
    isc_quad_low: Cardinal;
  end;
  PISC_QUAD = ^ISC_QUAD;

  TISC_QUAD = ISC_QUAD;

{*********************}
{* InterBase handles *}
{*********************}

  Tisc_att_handle = Pointer;        pisc_att_handle = ^Tisc_att_handle;
	Tisc_blob_handle = Pointer;       pisc_blob_handle = ^Tisc_blob_handle;
  Tisc_db_handle = Pointer;         pisc_db_handle = ^Tisc_db_handle;
  Tisc_req_handle = Pointer;        pisc_req_handle = ^Tisc_req_handle;
  Tisc_stmt_handle = Pointer;       pisc_stmt_handle = ^Tisc_stmt_handle;
	Tisc_svc_handle = Pointer;        pisc_svc_handle = ^Tisc_svc_handle;
  Tisc_tr_handle = Pointer;         pisc_tr_handle = ^Tisc_tr_handle;

  Tisc_callback = procedure( ptr: Pointer; length: Word; updated: PChar);

//*******************/
//* Exception Class */
//*******************/

EIBError = class(Exception);

{*****************}
{* Status vector *}
{*****************}

  ISC_STATUS = ISC_LONG;                PISC_STATUS = ^ISC_STATUS;
  
  ISC_STATUS_VECTOR = array[0..19] of ISC_STATUS;
  
  PSTATUS_VECTOR = ^ISC_STATUS_VECTOR;  PPSTATUS_VECTOR = ^PSTATUS_VECTOR;

{********************}
{* Array structures *}
{********************}

  ISC_ARRAY_BOUND = record
    array_bound_lower: Word;
    array_bound_upper: Word;
  end;

  ISC_ARRAY_DESC = record
    array_desc_dtype: Byte;
	  array_desc_scale: smallint;
    array_desc_length: Byte;
		array_desc_field_name: array [0..31] of Char;
    array_desc_relation_name: array [0..31] of Char;
    array_desc_dimensions: Word;
    array_desc_flags: Word;
    array_desc_bounds: array [0..15] of ISC_ARRAY_BOUND;
  end;
  PISC_ARRAY_DESC = ^ISC_ARRAY_DESC;

{*******************}
{* BLOb structures *}
{*******************}

	ISC_BLOB_DESC = record
    SubType: Smallint;
    Charset: Smallint;
    SegmentSize: Smallint;
    ColumnName: array [0..31] of Char;
    TableName: array [0..31] of Char;
  end;

  PISC_BLOB_DESC = ^ISC_BLOB_DESC;

  BSTREAM = record
    bstr_blob: Pointer;         {* BLOb handle *}
	  bstr_buffer: PChar;         {* Address of buffer *}
    bstr_ptr: PChar;            {* Next character *}
		bstr_length: Smallint;      {* Length of buffer *}
    bstr_cnt: Smallint;         {* Characters in buffer *}
    bstr_mode: Char;            {* OUTPUT or INPUT *}
  end;

  ISC_BLOB_CTL = record
    ctl_source: PISC_STATUS;          {* Source filter *}
    ctl_source_handle: Pointer;       {* Argument to pass to source *}
    ctl_to_sub_type: Smallint;        {* Target type *}
    ctl_from_sub_type: Smallint;      {* Source type *}
	  ctl_buffer_length: Byte;          {* Length of buffer *}
    ctl_segment_length: Byte;         {* Length of current segment *}
    ctl_bpb_length: Byte;             {* Length of BLOb Parameter Buffer *}
    ctl_bpb: PChar;                   {* Address of BLOb Parameter Buffer *}
    ctl_buffer: PByte;                {* Address of segment buffer *}
    ctl_max_segment: ISC_LONG;        {* Length of longest segment *}
    ctl_number_segments: ISC_LONG;    {* Total number of segments *}
    ctl_total_length: ISC_LONG;       {* Total length of BLOb *}
    ctl_status: PSTATUS_VECTOR;       {* Address of status vector *}
    ctl_data: array [0..7] of Longint;{* Application specific data *}
  end;
  PISC_BLOB_CTL = ^ISC_BLOB_CTL;

{**************************}
{* Dynamic SQL structures *}
{**************************}

  SQLVAR = record
    sqltype: Smallint;
    sqllen: Smallint;
    sqldata: Pointer;
    sqlind: ^Smallint;
    sqlname_len: Smallint;
    sqlname: array [0..29] of Char;
  end;
	PSQLVAR = ^SQLVAR;

  SQLDA = record
    sqldaid: array [0..7] of Char;
    sqldabc: ISC_LONG;
    sqln: Smallint;
    sqld: Smallint;
    sqlvar: array [0..0] of SQLVAR;
  end;
  PSQLDA = ^SQLDA;


  XSQLVAR = record
    sqltype: Smallint;                  {* datatype of field *}
    sqlscale: Smallint;                 {* scale factor *}
    sqlsubtype: Smallint;               {* datatype subtype - BLObs & Text types only *}
    sqllen: Smallint;                   {* length of data area *}
    sqldata: Pointer;                   {* address of data *}
    sqlind: ^Smallint;                  {* address of indicator variable *}
    sqlname_length: Smallint;           {* length of sqlname field *}
    sqlname: array [0..31] of Char;     {* name of field, name length + space for NULL *}
		relname_length: Smallint;           {* length of relation name *}
    relname: array [0..31] of Char;     {* field's relation name + space for NULL *}
    ownname_length: Smallint;           {* length of owner name *}
    ownname: array [0..31] of Char;     {* relation's owner name + space for NULL *}
    aliasname_length: Smallint;         {* length of alias name *}
    aliasname: array [0..31] of Char;   {* relation's alias name + space for NULL *}
  end;
  PXSQLVAR = ^XSQLVAR;

	XSQLDA = record
    version		: Smallint;                  		{* version of this XSQLDA *}
    sqldaid		: array [0..7] of Char;      		{* XSQLDA name field *}
    sqldabc		: isc_long;                  		{* length in bytes of SQLDA *}
    sqln			: Smallint;                     {* number of fields allocated *}
    sqld			: Smallint;                     {* actual number of fields *}
    sqlvar		: array [0..0] of XSQLVAR;    	{* first field address *}
  end;
  PXSQLDA = ^XSQLDA;

{********************************
* Transaction Existence Buffer *
********************************}

	ISC_TEB = record
	  db_ptr: pisc_db_handle;
	  tpb_len: Longint;
	  tpb_ptr: PChar;
	end;
	PISC_TEB = ^ISC_TEB;

{****************************}
{* Date and Time structures *}
{****************************}

  TM = record
	  sec     : Integer; { seconds (0-59) }
    min     : Integer; { minutes (0-59) }
    hour    : Integer; { hour (0-23) }
    mday    : Integer; { day of month (1-31) }
    mon     : Integer; { month of year (0-11) }
    year    : Integer; { year (year - 1900 ) }
    wday    : Integer; { day of week (0-6  Sunday=0)}
		yday    : Integer; { day of year (0-364) }
		isdst   : Integer; { daylight savings in effect (1 = True) }
  end;
	PTM = ^TM;
  


	DATETIME = record
	  IbDateTime: ISC_QUAD;
	  TimeDate: PTM;
  end;

const
  // C Consts
  cYearOffset     = 1900;

{***************************}
{* OSRI Database functions *}
{***************************}
    function isc_array_get_slice(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        array_id: PISC_QUAD;
        desc: PISC_ARRAY_DESC;
        dest_array: Pointer;
        slice_length: PISC_LONG
    ): ISC_STATUS; stdcall;

    function isc_array_lookup_bounds(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        table_name: PChar;
        column_name: PChar;
        desc: PISC_ARRAY_DESC
    ): ISC_STATUS; stdcall;

    function isc_array_lookup_desc(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        table_name: PChar;
        column_name: PChar;
        desc: PISC_ARRAY_DESC
    ): ISC_STATUS; stdcall;

    function isc_array_put_slice(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        array_id: PISC_QUAD;
        desc: PISC_ARRAY_DESC;
        source_array: Pointer;
        slice_length: PISC_LONG
    ): ISC_STATUS; stdcall;

    function isc_array_set_desc(
        status_vector: PSTATUS_VECTOR;
        table_name: PChar;
        column_name: PChar;
        sql_dtype: PSmallint;
        sql_length: PSmallint;
        dimensions: PSmallint;
        desc: PISC_ARRAY_DESC
    ): ISC_STATUS; stdcall;

    function isc_attach_database(
        status_vector: PSTATUS_VECTOR;
        db_name_length: Short;
        db_name: PChar;
        db_handle: pisc_db_handle;
        parm_buffer_length: Short;
        parm_buffer: PChar
    ): ISC_STATUS; stdcall;

    procedure isc_blob_default_desc(
        desc: ISC_BLOB_DESC;
        table_name: PChar;
        column_name: PChar
    ); stdcall;

    function isc_blob_gen_bpb(
        status_vector: PSTATUS_VECTOR;
        to_desc: PISC_BLOB_DESC;
        from_desc: PISC_BLOB_DESC;
        bpb_buffer_length: Byte;
        bpb_buffer: Pointer;
        bpb_length: PByte
    ): ISC_STATUS; stdcall;

    function isc_blob_info(
        status_vector: PSTATUS_VECTOR;
        blob_handle: pisc_blob_handle;
        item_list_buffer_length: Smallint;
        item_list_buffer: Pointer;
        result_buffer_length: Smallint;
        result_buffer: Pointer
    ): ISC_STATUS; stdcall;

    function isc_blob_lookup_desc(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        tablename: PChar;
        colname: PChar;
        blobdesc: ISC_BLOB_DESC;
        global: PChar
    ): ISC_STATUS; stdcall;

    function isc_blob_set_desc(
        status_vector: PSTATUS_VECTOR;
        table_name: PChar;
        column_name: PChar;
        subtype: Smallint;
        charset: Smallint;
        segment_size: Smallint;
        desc: PISC_BLOB_DESC
    ): ISC_STATUS; stdcall;

    function isc_cancel_blob(
        status_vector: PSTATUS_VECTOR;
        blob_handle: pisc_blob_handle
    ): ISC_STATUS; stdcall;

    function isc_cancel_events(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        event_id: pisc_long
    ): ISC_STATUS; stdcall;

    function isc_close_blob(
        status_vector: PSTATUS_VECTOR;
        blob_handle: pisc_blob_handle
    ): ISC_STATUS; stdcall;

    function isc_commit_retaining(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle
    ): ISC_STATUS; stdcall;

    function isc_commit_transaction(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle
    ): ISC_STATUS; stdcall;

    function isc_create_blob(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        blob_handle: pisc_blob_handle;
        pblob_id : PISC_QUAD
    ): ISC_STATUS; stdcall;

    function isc_create_blob2(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        blob_handle: pisc_blob_handle;
        pblob_id: PISC_QUAD;
        bpb_length: Smallint;
        bpb_address: PChar
    ): ISC_STATUS; stdcall;

    function isc_database_info(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        item_list_buffer_length: Smallint;
        item_list_buffer: Pointer;
        result_buffer_length: Smallint;
        result_buffer: Pointer
    ): ISC_STATUS; stdcall;

    procedure isc_decode_date(
        ib_date: PISC_QUAD;
        tm_date: PTM
    ); stdcall;

    function isc_detach_database(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle
    ): ISC_STATUS; stdcall;

    function isc_drop_database(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle
    ): ISC_STATUS; stdcall;

    function isc_dsql_allocate_statement(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        stmt_handle: pisc_stmt_handle
    ): ISC_STATUS; stdcall;

    function isc_dsql_allocate_statement2(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        stmt_handle: pisc_stmt_handle
    ): ISC_STATUS; stdcall;

    function isc_dsql_describe(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_describe_bind(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_execute(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        stmt_handle: pisc_stmt_handle;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_execute2(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        stmt_handle: pisc_stmt_handle;
        dialect: Word;
        in_xsqlda: PXSQLDA;
        out_xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_execute_immediate(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        length: Word;
        statement: PChar;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_exec_immed2(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        length: Word;
        statement: PChar;
        dialect: Word;
        in_xsqlda: PXSQLDA;
        out_xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_fetch(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_finish(
        db_handle: pisc_db_handle
    ): ISC_STATUS; stdcall;

    function isc_dsql_free_statement(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        option: Word
    ): ISC_STATUS; stdcall;

    function isc_dsql_prepare(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        stmt_handle: pisc_stmt_handle;
        length: Word;
        statement: PChar;
        dialect: Word;
        xsqlda: PXSQLDA
    ): ISC_STATUS; stdcall;

    function isc_dsql_set_cursor_name(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        cursor_name: PChar;
        cursor_type: Word
    ): ISC_STATUS; stdcall;

    function isc_dsql_sql_info(
        status_vector: PSTATUS_VECTOR;
        stmt_handle: pisc_stmt_handle;
        item_length: Word;
        items: PChar;
        buffer_length: Word;
        buffer: PChar
    ): ISC_STATUS; stdcall;

    procedure isc_encode_date(
        tm_date: PTM;
        ib_date: PISC_QUAD
    ); stdcall;

    function isc_event_block(
        event_buffer: Pointer;
        result_buffer: Pointer;
        count: Short;
        name1: PChar
    ): Longint; cdecl;

    function isc_event_block_asm: longint; stdcall;

    procedure isc_event_counts(
        status_vector: PSTATUS_VECTOR;
        buffer_length: Word;
        event_buffer: PChar;
        result_buffer: PChar
    ); stdcall;

    function isc_free(
        buffer: PChar
    ): isc_long; stdcall;

    function isc_get_segment(
        status_vector: PSTATUS_VECTOR;
        blob_handle: pisc_blob_handle;
        actual_seg_length : PWord;
        seg_buffer_length : Word;
        seg_buffer : Pointer
    ): ISC_STATUS; stdcall;

    function isc_interprete(
        buffer: PChar;
        status_vector_ptr: PPSTATUS_VECTOR
    ): ISC_STATUS; stdcall;

    function isc_open_blob2(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        trans_handle: pisc_tr_handle;
        blob_handle: pisc_blob_handle;
        blob_id : PISC_QUAD;
        bpb_length : Word;
        bpb_address : Pointer
    ): ISC_STATUS; stdcall;

    function isc_prepare_transaction(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle
    ): ISC_STATUS; stdcall;

    function isc_prepare_transaction2(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        msg_length: Word;
        msg: PChar
    ): ISC_STATUS; stdcall;

    procedure isc_print_status(
        status_vector_ptr: PSTATUS_VECTOR
    );

    procedure isc_print_sqlerror(
        SQLCODE: ISC_LONG;
        status_vector_ptr: PSTATUS_VECTOR
    );

    function isc_put_segment(
        status_vector: PSTATUS_VECTOR;
        blob_handle: pisc_blob_handle;
        seg_buffer_length : Word;
        seg_buffer_address : Pointer
    ): ISC_STATUS; stdcall;

    function isc_que_events(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        event_id: PISC_LONG;
        length: Word;
        event_buffer: PChar;
        event_function: Tisc_callback;
        event_function_arg: Pointer
    ): ISC_STATUS; stdcall;

    function isc_rollback_transaction(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle
    ): ISC_STATUS; stdcall;

    function isc_sqlcode(
        status_vector: PSTATUS_VECTOR
    ): ISC_LONG; stdcall;

    procedure isc_sql_interprete(
        sqlcode: ISC_LONG;
        buffer: PChar;
        buffer_length: Short
    ); stdcall;

    function isc_start_multiple(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        db_handle_count: Short;
        teb_vector_address: PISC_TEB
    ): ISC_STATUS; stdcall;

    function isc_start_transaction(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        db_handle_count: Short;
        db_handle: pisc_db_handle;
        tpb_length: Word;
        tpb_addr: PChar
    ): ISC_STATUS; stdcall;

    function isc_transaction_info(
        status_vector: PSTATUS_VECTOR;
        trans_handle: pisc_tr_handle;
        item_list_buffer_length: Smallint;
        item_list_buffer: Pointer;
        result_buffer_length: Smallint;
        result_buffer: Pointer
    ): ISC_STATUS; stdcall;

    function isc_version(
        db_handle: pisc_db_handle;
        function_name: Tisc_callback;
        user_arg: Pointer
    ): Integer; stdcall;

    function isc_wait_for_event(
        status_vector: PSTATUS_VECTOR;
        db_handle: pisc_db_handle;
        length: Short;
        event_buffer: PChar;
        result_buffer: PChar
    ): ISC_STATUS; stdcall;


    function SQLDA_LENGTH(n: Word): Longint;

    function XSQLDA_LENGTH(n: Word): Longint;

implementation

function SQLDA_LENGTH(n: Word): Longint;
begin
    SQLDA_LENGTH := (SizeOf(SQLDA) + (n - 1) * SizeOf(SQLVAR));
end;

function XSQLDA_LENGTH(n: Word): Longint;
begin
    XSQLDA_LENGTH := (SizeOf(XSQLDA) + (n - 1) * SizeOf(XSQLVAR));
end;

procedure isc_print_status(status_vector_ptr: PSTATUS_VECTOR);
var
  msgBuffer: array[0..511] of Char;
  errMsg: String;
  lastMsg: String;
  errCode: ISC_STATUS;
begin
  errMsg := 'InterBase error:' + #13#10;
  repeat
     errCode := isc_interprete(@msgBuffer, @status_vector_ptr);
     if lastMsg <> StrPas(msgBuffer) then
     begin
        lastMsg := StrPas(msgBuffer);
        if Length(errMsg) <> 0 then
           errMsg := errMsg + #13#10;
        errMsg := errMsg + lastMsg;
     end;
  until errCode = 0;
  raise EIBError.Create(errMsg);
end;

procedure isc_print_sqlerror(SQLCODE: ISC_LONG; status_vector_ptr: PSTATUS_VECTOR);
var
  msgBuffer: array[0..511] of Char;
  errMsg: String;
  lastMsg: String;
begin
  errMsg := 'InterBase SQL error (' + IntToStr(SQLCODE) + '):' + #13#10;
  isc_sql_interprete(SQLCODE, @msgBuffer, SizeOf(msgBuffer));
  if lastMsg <> StrPas(msgBuffer) then
  begin
    lastMsg := StrPas(msgBuffer);
    if Length(errMsg) <> 0 then
      errMsg := errMsg + #13#10;
    errMsg := errMsg + lastMsg;
  end;
  raise EIBError.Create(errMsg);
end;

function isc_array_get_slice; external IBASE_DLL name 'isc_array_get_slice';

function isc_array_lookup_bounds;  external IBASE_DLL name 'isc_array_lookup_bounds';

function isc_array_lookup_desc; external IBASE_DLL name 'isc_array_lookup_desc';

function isc_array_put_slice; external IBASE_DLL name 'isc_array_put_slice';

function isc_array_set_desc; external IBASE_DLL name 'isc_array_set_desc';

function isc_attach_database; external IBASE_DLL name 'isc_attach_database';

procedure isc_blob_default_desc; external IBASE_DLL name 'isc_blob_default_desc';

function isc_blob_gen_bpb; external IBASE_DLL name 'isc_blob_gen_bpb';

function isc_blob_info;  external IBASE_DLL name 'isc_blob_info';

function isc_blob_lookup_desc; external IBASE_DLL name 'isc_blob_lookup_desc';

function isc_blob_set_desc; external IBASE_DLL name 'isc_blob_set_desc';

function isc_cancel_events; external IBASE_DLL name 'isc_cancel_events';

function isc_close_blob; external IBASE_DLL name 'isc_close_blob';

function isc_cancel_blob; external IBASE_DLL name 'isc_cancel_blob';

function isc_commit_retaining; external IBASE_DLL name 'isc_commit_retaining';

function isc_commit_transaction; external IBASE_DLL name 'isc_commit_transaction';

function isc_create_blob; external IBASE_DLL name 'isc_create_blob';

function isc_create_blob2; external IBASE_DLL name 'isc_create_blob2';

function isc_database_info; external IBASE_DLL name 'isc_database_info';

function isc_detach_database; external IBASE_DLL name 'isc_detach_database';

procedure isc_decode_date; external IBASE_DLL name 'isc_decode_date';

function isc_dsql_execute_immediate; external IBASE_DLL name 'isc_dsql_execute_immediate';

function isc_drop_database; external IBASE_DLL name 'isc_drop_database';

function isc_sqlcode; external IBASE_DLL name 'isc_sqlcode';

function isc_dsql_allocate_statement; external IBASE_DLL name 'isc_dsql_allocate_statement';

function isc_dsql_allocate_statement2; external IBASE_DLL name 'isc_dsql_allocate_statement2';

function isc_dsql_describe; external IBASE_DLL name 'isc_dsql_describe';

function isc_dsql_describe_bind; external IBASE_DLL name 'isc_dsql_describe_bind';

function isc_dsql_exec_immed2; external IBASE_DLL name 'isc_dsql_exec_immed2';

function isc_dsql_execute; external IBASE_DLL name 'isc_dsql_execute';

function isc_dsql_execute2; external IBASE_DLL name 'isc_dsql_execute2';

function isc_dsql_fetch; external IBASE_DLL name 'isc_dsql_fetch';

function isc_dsql_free_statement; external IBASE_DLL name 'isc_dsql_free_statement';

function isc_dsql_prepare; external IBASE_DLL name 'isc_dsql_prepare';

function isc_dsql_set_cursor_name; external IBASE_DLL name 'isc_dsql_set_cursor_name';

function isc_dsql_sql_info; external IBASE_DLL name 'isc_dsql_sql_info';

procedure isc_encode_date; external IBASE_DLL name 'isc_encode_date';

function isc_event_block; external IBASE_DLL name 'isc_event_block';

function isc_event_block_asm; external IBASE_DLL name 'isc_event_block_asm';

procedure isc_event_counts; external IBASE_DLL name 'isc_event_counts';

function isc_finish; external IBASE_DLL name 'isc_finish';

function isc_free; external IBASE_DLL name 'isc_free';

function isc_get_segment; external IBASE_DLL name 'isc_get_segment';

function isc_interprete; external IBASE_DLL name 'isc_interprete';

function isc_que_events; external IBASE_DLL name 'isc_event_block_asm';

function isc_open_blob2; external IBASE_DLL name 'isc_open_blob2';

function isc_prepare_transaction; external IBASE_DLL name 'isc_prepare_transaction';

function isc_prepare_transaction2; external IBASE_DLL name 'isc_prepare_transaction2';

function isc_put_segment; external IBASE_DLL name 'isc_put_segment';

function isc_rollback_transaction; external IBASE_DLL name 'isc_rollback_transaction';

procedure isc_sql_interprete; external IBASE_DLL name 'isc_sql_interprete';

function isc_start_multiple; external IBASE_DLL name 'isc_start_multiple';

function isc_start_transaction; external IBASE_DLL name 'isc_start_transaction';

function isc_transaction_info; external IBASE_DLL name 'isc_transaction_info';

function isc_version; external IBASE_DLL name 'isc_version';

function isc_wait_for_event; external IBASE_DLL name 'isc_wait_for_event';


end.




